@@ -87,8 +87,6 @@ struct _MountAuthData {
 
 };
 
-
-
 struct _GVfsBackendDav
 {
   GVfsBackendHttp parent_instance;
@@ -118,7 +116,7 @@ g_vfs_backend_dav_init (GVfsBackendDav *backend)
 }
 
 /* ************************************************************************* */
-/*  */
+/* Small utility functions */
 
 static inline gboolean
 sm_has_header (SoupMessage *msg, const char *header)
@@ -579,6 +577,16 @@ parse_resourcetype (xmlNodePtr rt)
   return type;
 }
 
+static inline void
+file_info_set_content_type (GFileInfo *info, const char *type)
+{
+  g_file_info_set_content_type (info, type);
+  g_file_info_set_attribute_string (info,
+                                    G_FILE_ATTRIBUTE_STANDARD_FAST_CONTENT_TYPE,
+                                    type);
+
+}
+
 static void
 ms_response_to_file_info (MsResponse *response,
                           GFileInfo  *info)
@@ -590,11 +598,16 @@ ms_response_to_file_info (MsResponse *response,
   char       *basename;
   const char *text;
   GTimeVal    tv;
+  GFileType   file_type;
+  char       *mime_type;
+  GIcon      *icon;
 
   basename = ms_response_get_basename (response);
   g_file_info_set_name (info, basename);
   g_file_info_set_edit_name (info, basename);
-  g_free (basename);
+
+  file_type = G_FILE_TYPE_UNKNOWN;
+  mime_type = NULL;
 
   ms_response_get_propstat_iter (response, &iter);
   while (xml_node_iter_next (&iter))
@@ -613,8 +626,8 @@ ms_response_to_file_info (MsResponse *response,
 
           if (node_has_name (node, "resourcetype"))
             {
-              GFileType type = parse_resourcetype (node);
-              g_file_info_set_file_type (info, type);
+              file_type = parse_resourcetype (node);
+              g_file_info_set_file_type (info, file_type);
             }
           else if (node_has_name (node, "displayname"))
             {
@@ -636,7 +649,7 @@ ms_response_to_file_info (MsResponse *response,
             }
           else if (node_has_name (node, "getcontenttype"))
             {
-              g_file_info_set_content_type (info, text);
+              mime_type = g_strdup (text);
             }
           else if (node_has_name (node, "getcontentlength"))
             {
@@ -651,6 +664,30 @@ ms_response_to_file_info (MsResponse *response,
             }
         }
     }
+
+  if (file_type == G_FILE_TYPE_DIRECTORY)
+    {
+      icon = g_themed_icon_new ("folder");
+      file_info_set_content_type (info, "inode/directory");
+    }
+  else
+    {
+      if (mime_type == NULL)
+        mime_type = g_content_type_guess (basename, NULL, 0, NULL);
+
+      icon = g_content_type_get_icon (mime_type);
+
+      if (G_IS_THEMED_ICON (icon))
+        g_themed_icon_append_name (G_THEMED_ICON (icon), "text-x-generic");
+
+      file_info_set_content_type (info, mime_type);
+    }
+
+  g_file_info_set_icon (info, icon);
+  g_object_unref (icon);
+  g_free (mime_type);
+  g_free (basename);
+
 }
 
 static GFileType
@@ -741,14 +778,18 @@ propfind_request_new (GVfsBackend     *backend,
   if (properties != NULL)
     {
       const PropName *prop;
+      g_string_append (body, PROPSTAT_XML_PROP_BEGIN);
 
       for (prop = properties; prop->name; prop++)
         {
           if (prop->namespace != NULL)
-            g_string_append (body, "<%s xmlns=\"%s\"/>");
+            g_string_append_printf (body, "<%s xmlns=\"%s\"/>\n",
+                                    prop->name,
+                                    prop->namespace);
           else
-            g_string_append (body, "<D:%s/>");
+            g_string_append_printf (body, "<D:%s/>\n", prop->name);
         }
+      g_string_append (body, PROPSTAT_XML_PROP_END);
     }
   else
     g_string_append (body, PROPSTAT_XML_ALLPROP);
@@ -907,7 +948,7 @@ stat_location (GVfsBackend  *backend,
 
 
 /* ************************************************************************* */
-/*  */
+/* Authentication */
 
 static void
 mount_auth_info_free (MountAuthData *data)
@@ -1100,6 +1141,9 @@ keyring_save_authinfo (AuthInfo *info,
                                info->pw_save);
 }
 
+/* ************************************************************************* */
+/* Authentication */
+
 static inline GMountSpec *
 g_mount_spec_dup_known (GMountSpec *spec)
 {
@@ -1146,6 +1190,7 @@ do_mount (GVfsBackend  *backend,
   gboolean        is_webdav;
   gboolean        res;
   char           *last_good_path;
+  char           *display_name;
 
   g_print ("+ mount\n");
 
@@ -1270,7 +1315,11 @@ do_mount (GVfsBackend  *backend,
 
   g_vfs_backend_set_mount_spec (backend, mount_spec);
   g_vfs_backend_set_icon_name (backend, "folder-remote");
-
+  
+  display_name = g_strdup_printf (_("WebDAV on %s"), host);
+  g_vfs_backend_set_display_name (backend, display_name);
+  g_free (display_name);
+  
   /* cleanup */
   g_mount_spec_unref (mount_spec);
   g_object_unref (msg_opts);
@@ -1291,6 +1340,17 @@ do_mount (GVfsBackend  *backend,
   g_print ("- mount\n");
 }
 
+static PropName ls_propnames[] = {
+    {"creationdate",     NULL},
+    {"displayname",      NULL},
+    {"getcontentlength", NULL},
+    {"getcontenttype",   NULL},
+    {"getetag",          NULL},
+    {"getlastmodified",  NULL},
+    {"resourcetype",     NULL},
+    {NULL,               NULL}
+};
+
 /* *** query_info () *** */
 static void
 do_query_info (GVfsBackend           *backend,
@@ -1303,14 +1363,12 @@ do_query_info (GVfsBackend           *backend,
   SoupMessage *msg;
   Multistatus  ms;
   xmlNodeIter  iter;
-  SoupURI     *base;
   gboolean     res;
   GError      *error;
 
-  base    = G_VFS_BACKEND_HTTP (backend)->mount_base;
   error   = NULL;
 
-  msg = propfind_request_new (backend, filename, 0, NULL);
+  msg = propfind_request_new (backend, filename, 0, ls_propnames);
 
   if (msg == NULL)
     {
@@ -1377,13 +1435,11 @@ do_enumerate (GVfsBackend           *backend,
   Multistatus  ms;
   xmlNodeIter  iter;
   gboolean     res;
-  SoupURI     *base;
   GError      *error;
  
-  base  = G_VFS_BACKEND_HTTP (backend)->mount_base;
   error = NULL;
 
-  msg = propfind_request_new (backend, filename, 1, NULL);
+  msg = propfind_request_new (backend, filename, 1, ls_propnames);
 
   if (msg == NULL)
     {
@@ -1413,14 +1469,11 @@ do_enumerate (GVfsBackend           *backend,
   while (xml_node_iter_next (&iter))
     {
       MsResponse  response;
-      const char *basename;
       GFileInfo  *info;
 
       if (! multistatus_get_response (&iter, &response))
         continue;
 
-      basename = ms_response_get_basename (&response);
-
       if (ms_response_is_target (&response))
         continue;
 
@@ -1463,7 +1516,10 @@ try_create_tested_existence (SoupSession *session, SoupMessage *msg,
   uri = soup_message_get_uri (msg);
   put_msg = soup_message_new_from_uri (SOUP_METHOD_PUT, uri);
 
-  soup_message_headers_append (put_msg->request_headers, "If-None-Match", "*");
+  /* 
+   * Doesn't work with apache > 2.2.9
+   * soup_message_headers_append (put_msg->request_headers, "If-None-Match", "*");
+   */
   stream = soup_output_stream_new (op_backend->session, put_msg, -1);
   g_object_unref (put_msg);
 
@@ -1647,6 +1703,7 @@ close_write_ready (GObject      *source_object,
   GError        *error;
   gboolean       res;
 
+  error = NULL;
   job = G_VFS_JOB (user_data);
   stream = G_OUTPUT_STREAM (source_object);
   res = g_output_stream_close_finish (stream,
@@ -1720,7 +1777,6 @@ do_delete (GVfsBackend   *backend,
            GVfsJobDelete *job,
            const char    *filename)
 {
-  SoupSession *session;
   SoupMessage *msg;
   SoupURI     *uri;
   GFileType    file_type;
@@ -1730,7 +1786,6 @@ do_delete (GVfsBackend   *backend,
   GError      *error;
 
   error = NULL;
-  session = G_VFS_BACKEND_HTTP (backend)->session;
 
   uri = http_backend_uri_for_filename (backend, filename, FALSE);
   res = stat_location (backend, uri, &file_type, &num_children, &error);
@@ -1766,6 +1821,7 @@ do_delete (GVfsBackend   *backend,
   g_object_unref (msg);
 }
 
+
 /* ************************************************************************* */
 /*  */
 
