@@ -35,7 +35,6 @@
 #include <gio/gio.h>
 
 #include <libsoup/soup.h>
-
 #include "gvfsbackendhttp.h"
 #include "gvfsjobopenforread.h"
 #include "gvfsjobread.h"
@@ -51,12 +50,9 @@
 #include "gvfsdaemonprotocol.h"
 
 #include "soup-input-stream.h"
-#include "soup-output-stream.h"
 
 
-
-
-G_DEFINE_TYPE (GVfsBackendHttp, g_vfs_backend_http, G_VFS_TYPE_BACKEND);
+G_DEFINE_TYPE (GVfsBackendHttp, g_vfs_backend_http, G_VFS_TYPE_BACKEND)
 
 static void
 g_vfs_backend_http_finalize (GObject *object)
@@ -71,6 +67,10 @@ g_vfs_backend_http_finalize (GObject *object)
   soup_session_abort (backend->session);
   g_object_unref (backend->session);
 
+  soup_session_abort (backend->session_async);
+  g_object_unref (backend->session_async);
+
+
   if (G_OBJECT_CLASS (g_vfs_backend_http_parent_class)->finalize)
     (*G_OBJECT_CLASS (g_vfs_backend_http_parent_class)->finalize) (object);
 }
@@ -80,14 +80,25 @@ g_vfs_backend_http_init (GVfsBackendHttp *backend)
 {
   g_vfs_backend_set_user_visible (G_VFS_BACKEND (backend), FALSE);  
 
-  backend->session = soup_session_async_new ();
+  backend->session = soup_session_sync_new_with_options ("user-agent",
+                                                         "gvfs/" VERSION,
+                                                         NULL);
+
+  backend->session_async = soup_session_async_new_with_options ("user-agent",
+                                                                "gvfs/" VERSION,
+                                                                NULL);
 }
 
-SoupUri *
-g_vfs_backend_uri_for_filename (GVfsBackend *backend, const char *filename)
+/* ************************************************************************* */
+/* public utility functions */
+
+SoupURI *
+http_backend_uri_for_filename (GVfsBackend *backend,
+                                const char  *filename,
+                                gboolean     is_dir)
 {
   GVfsBackendHttp *op_backend;
-  SoupUri         *uri;
+  SoupURI         *uri;
   char            *path;
 
   op_backend = G_VFS_BACKEND_HTTP (backend);
@@ -99,14 +110,138 @@ g_vfs_backend_uri_for_filename (GVfsBackend *backend, const char *filename)
 
   /* Otherwise, we append filename to mount_base (which is assumed to
    * be a directory in this case).
+   *
+   * Add a "/" in cases where it is likely that the url is going
+   * to be a directory to avoid redirections
    */
-  path = g_build_path ("/", uri->path, filename, NULL);
+  if (is_dir == FALSE || g_str_has_suffix (filename, "/"))
+    path = g_build_path ("/", uri->path, filename, NULL);
+  else
+    path = g_build_path ("/", uri->path, filename, "/", NULL);
+
   g_free (uri->path);
-  uri->path = path;
+  uri->path = g_uri_escape_string (path, G_URI_RESERVED_CHARS_ALLOWED_IN_PATH,
+                                   FALSE);
+  g_free (path);
 
   return uri;
 }
 
+char *
+http_uri_get_basename (const char *uri_str)
+{
+    const char *parent;
+    const char *path;
+    char       *to_free;
+    char       *basename;
+    size_t      len;
+
+    if (uri_str == NULL || *uri_str == '\0')
+      return NULL;
+
+    path =  uri_str;
+
+    /* remove any leading slashes */
+    while (*path == '/' || *path == ' ')
+        path++;
+
+    len = strlen (path);
+
+    if (len == 0)
+      return g_strdup ("/");
+
+    /* remove any trailing slashes */
+    while (path[len - 1] == '/' || path[len - 1] == ' ')
+        len--;
+
+    parent = g_strrstr_len (path, len, "/");
+
+    if (parent)
+      {
+        parent++; /* skip the found / char */
+        to_free = g_strndup (parent, (len - (parent - path)));
+      }
+    else
+      to_free = g_strndup (path, len);
+
+    basename = soup_uri_decode (to_free);
+    g_free (to_free);
+
+    return basename;
+}
+
+guint
+http_error_code_from_status (guint status)
+{
+  switch (status) {
+
+  case SOUP_STATUS_CANT_RESOLVE:
+  case SOUP_STATUS_CANT_RESOLVE_PROXY:
+    return G_IO_ERROR_HOST_NOT_FOUND;
+
+  case SOUP_STATUS_CANCELLED:
+    return G_IO_ERROR_CANCELLED;
+
+  case SOUP_STATUS_UNAUTHORIZED:
+  case SOUP_STATUS_PAYMENT_REQUIRED:
+  case SOUP_STATUS_FORBIDDEN:
+    return G_IO_ERROR_PERMISSION_DENIED;
+
+  case SOUP_STATUS_NOT_FOUND:
+  case SOUP_STATUS_GONE:
+    return G_IO_ERROR_NOT_FOUND;    
+
+  case SOUP_STATUS_GATEWAY_TIMEOUT:
+    return G_IO_ERROR_TIMED_OUT;
+  }
+
+  return G_IO_ERROR_FAILED;
+}
+
+
+static void
+g_vfs_job_failed_from_http_status (GVfsJob *job, guint status_code, const char *message)
+{
+  switch (status_code) {
+
+  case SOUP_STATUS_NOT_FOUND:
+    g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_NOT_FOUND,
+                      message);
+    break;
+
+  case SOUP_STATUS_UNAUTHORIZED:
+  case SOUP_STATUS_PAYMENT_REQUIRED:
+  case SOUP_STATUS_FORBIDDEN:
+    g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_PERMISSION_DENIED,
+                      _("HTTP Client Error: %s"), message);
+    break;
+  default:
+    g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_FAILED,
+                      _("HTTP Error: %s"), message);
+  }
+}
+
+guint
+http_backend_send_message (GVfsBackend *backend,
+                           SoupMessage *msg)
+{
+  GVfsBackendHttp *op_backend = G_VFS_BACKEND_HTTP (backend);
+  return soup_session_send_message (op_backend->session, msg);
+}
+
+void
+http_backend_queue_message (GVfsBackend         *backend,
+                            SoupMessage         *msg,
+                            SoupSessionCallback  callback,
+                            gpointer             user_data)
+{
+  GVfsBackendHttp *op_backend = G_VFS_BACKEND_HTTP (backend);
+  
+  soup_session_queue_message (op_backend->session_async, msg, 
+                              callback, user_data);
+}
+/* ************************************************************************* */
+/* virtual functions overrides */
 
 static gboolean
 try_mount (GVfsBackend  *backend,
@@ -117,7 +252,8 @@ try_mount (GVfsBackend  *backend,
 {
   GVfsBackendHttp *op_backend;
   const char      *uri_str;
-  SoupUri         *uri;
+  char            *path;
+  SoupURI         *uri;
   GMountSpec      *real_mount_spec;
 
   op_backend = G_VFS_BACKEND_HTTP (backend);
@@ -133,13 +269,22 @@ try_mount (GVfsBackend  *backend,
   if (uri == NULL)
     {
       g_vfs_job_failed (G_VFS_JOB (job),
-			G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT,
-			_("Invalid mount spec"));
+                        G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT,
+                        _("Invalid mount spec"));
       return TRUE;
     }
 
   real_mount_spec = g_mount_spec_new ("http");
   g_mount_spec_set (real_mount_spec, "uri", uri_str);
+
+  if (uri->path != NULL)
+    {
+      path = g_uri_unescape_string (uri->path, "/");
+      g_free (real_mount_spec->mount_prefix);
+      real_mount_spec->mount_prefix = g_mount_spec_canonicalize_path (path);
+      g_free (path);
+    }
+  
   g_vfs_backend_set_mount_spec (backend, real_mount_spec);
   
   op_backend->mount_base = uri;
@@ -167,7 +312,6 @@ open_for_read_ready (GObject      *source_object,
   res = soup_input_stream_send_finish (stream,
                                        result,
                                        &error);
-
   if (res == FALSE)
     {
       g_vfs_job_failed (G_VFS_JOB (job),
@@ -193,17 +337,15 @@ try_open_for_read (GVfsBackend        *backend,
 {
   GVfsBackendHttp *op_backend;
   GInputStream    *stream;
-  SoupUri         *uri;
   SoupMessage     *msg;
+  SoupURI         *uri;
 
   op_backend = G_VFS_BACKEND_HTTP (backend);
-  uri = g_vfs_backend_uri_for_filename (backend, filename);
-
+  uri = http_backend_uri_for_filename (backend, filename, FALSE);
   msg = soup_message_new_from_uri (SOUP_METHOD_GET, uri);
   soup_uri_free (uri);
-  soup_message_add_header (msg->request_headers, "User-Agent", "gvfs/" VERSION);
 
-  stream = soup_input_stream_new (op_backend->session, msg);
+  stream = soup_input_stream_new (op_backend->session_async, msg);
   g_object_unref (msg);
 
   soup_input_stream_send_async (stream,
@@ -254,10 +396,8 @@ try_read (GVfsBackend        *backend,
           char               *buffer,
           gsize               bytes_requested)
 {
-  GVfsBackendHttp *op_backend;
   GInputStream    *stream;
 
-  op_backend = G_VFS_BACKEND_HTTP (backend);
   stream = G_INPUT_STREAM (handle);
 
   g_input_stream_read_async (stream,
@@ -277,11 +417,9 @@ try_seek_on_read (GVfsBackend *backend,
                   goffset    offset,
                   GSeekType  type)
 {
-  GVfsBackendHttp *op_backend;
   GInputStream    *stream;
   GError          *error = NULL;
 
-  op_backend = G_VFS_BACKEND_HTTP (backend);
   stream = G_INPUT_STREAM (handle);
 
   if (!g_seekable_seek (G_SEEKABLE (stream), offset, type,
@@ -339,10 +477,8 @@ try_close_read (GVfsBackend       *backend,
                 GVfsJobCloseRead  *job,
                 GVfsBackendHandle  handle)
 {
-  GVfsBackendHttp *op_backend;
   GInputStream    *stream;
 
-  op_backend = G_VFS_BACKEND_HTTP (backend);
   stream = G_INPUT_STREAM (handle);
 
   g_input_stream_close_async (stream,
@@ -350,262 +486,121 @@ try_close_read (GVfsBackend       *backend,
                               G_VFS_JOB (job)->cancellable,
                               close_read_ready,
                               job);
-
   return TRUE;
 }
 
-/* *** create () *** */
-static void
-try_create_tested_existence (SoupMessage *msg, gpointer user_data)
-{
-  GVfsJob *job = G_VFS_JOB (user_data);
-  GVfsBackendHttp *op_backend = job->backend_data;
-  GOutputStream   *stream;
-  SoupMessage     *put_msg;
-
-  if (SOUP_STATUS_IS_SUCCESSFUL (msg->status_code))
-    {
-      g_vfs_job_failed (job,
-                        G_IO_ERROR,
-                        G_IO_ERROR_EXISTS,
-                        _("Target file already exists"));
-      return;
-    }
-  /* FIXME: other errors */
-
-  put_msg = soup_message_new_from_uri (SOUP_METHOD_PUT,
-                                       soup_message_get_uri (msg));
-  soup_message_add_header (put_msg->request_headers, "User-Agent", "gvfs/" VERSION);
-  soup_message_add_header (put_msg->request_headers, "If-None-Match", "*");
-  stream = soup_output_stream_new (op_backend->session, put_msg, -1);
-  g_object_unref (put_msg);
-
-  g_vfs_job_open_for_write_set_handle (G_VFS_JOB_OPEN_FOR_WRITE (job), stream);
-  g_vfs_job_succeeded (job);
-}  
-
-static gboolean
-try_create (GVfsBackend *backend,
-            GVfsJobOpenForWrite *job,
-            const char *filename,
-            GFileCreateFlags flags)
-{
-  GVfsBackendHttp *op_backend;
-  SoupUri         *uri;
-  SoupMessage     *msg;
-
-  /* FIXME: if SoupOutputStream supported chunked requests, we could
-   * use a PUT with "If-None-Match: *" and "Expect: 100-continue"
-   */
-
-  op_backend = G_VFS_BACKEND_HTTP (backend);
-  uri = g_vfs_backend_uri_for_filename (backend, filename);
-
-  msg = soup_message_new_from_uri (SOUP_METHOD_HEAD, uri);
-  soup_uri_free (uri);
-  soup_message_add_header (msg->request_headers, "User-Agent", "gvfs/" VERSION);
-
-  g_vfs_job_set_backend_data (G_VFS_JOB (job), op_backend, NULL);
-  soup_session_queue_message (op_backend->session, msg,
-                              try_create_tested_existence, job);
-  return TRUE;
-}
-
-/* *** replace () *** */
-static void
-open_for_replace_succeeded (GVfsBackendHttp *op_backend, GVfsJob *job,
-                            const SoupUri *uri, const char *etag)
-{
-  SoupMessage     *put_msg;
-  GOutputStream   *stream;
-
-  put_msg = soup_message_new_from_uri (SOUP_METHOD_PUT, uri);
-  soup_message_add_header (put_msg->request_headers, "User-Agent", "gvfs/" VERSION);
-  if (etag)
-    soup_message_add_header (put_msg->request_headers, "If-Match", etag);
-
-  stream = soup_output_stream_new (op_backend->session, put_msg, -1);
-  g_object_unref (put_msg);
-
-  g_vfs_job_open_for_write_set_handle (G_VFS_JOB_OPEN_FOR_WRITE (job), stream);
-  g_vfs_job_succeeded (job);
-}
-
-static void
-try_replace_checked_etag (SoupMessage *msg, gpointer user_data)
-{
-  GVfsJob *job = G_VFS_JOB (user_data);
-  GVfsBackendHttp *op_backend = job->backend_data;
-
-  if (msg->status_code == SOUP_STATUS_PRECONDITION_FAILED)
-    {
-      g_vfs_job_failed (G_VFS_JOB (job),
-                        G_IO_ERROR,
-                        G_IO_ERROR_WRONG_ETAG,
-                        _("The file was externally modified"));
-      return;
-    }
-  /* FIXME: other errors */
-
-  open_for_replace_succeeded (op_backend, job, soup_message_get_uri (msg),
-                              soup_message_get_header (msg->request_headers, "If-Match"));
-}  
-
-static gboolean
-try_replace (GVfsBackend *backend,
-             GVfsJobOpenForWrite *job,
-             const char *filename,
-             const char *etag,
-             gboolean make_backup,
-             GFileCreateFlags flags)
-{
-  GVfsBackendHttp *op_backend;
-  SoupUri         *uri;
-
-  /* FIXME: if SoupOutputStream supported chunked requests, we could
-   * use a PUT with "If-Match: ..." and "Expect: 100-continue"
-   */
-
-  op_backend = G_VFS_BACKEND_HTTP (backend);
-
-  if (make_backup)
-    {
-      g_vfs_job_failed (G_VFS_JOB (job),
-                        G_IO_ERROR,
-                        G_IO_ERROR_CANT_CREATE_BACKUP,
-                        _("Backup file creation failed"));
-      return TRUE;
-    }
-
-  uri = g_vfs_backend_uri_for_filename (backend, filename);
-
-  if (etag)
-    {
-      SoupMessage     *msg;
-
-      msg = soup_message_new_from_uri (SOUP_METHOD_HEAD, uri);
-      soup_uri_free (uri);
-      soup_message_add_header (msg->request_headers, "User-Agent", "gvfs/" VERSION);
-      soup_message_add_header (msg->request_headers, "If-Match", etag);
-
-      g_vfs_job_set_backend_data (G_VFS_JOB (job), op_backend, NULL);
-      soup_session_queue_message (op_backend->session, msg,
-                                  try_replace_checked_etag, job);
-      return TRUE;
-    }
-
-  open_for_replace_succeeded (op_backend, G_VFS_JOB (job), uri, NULL);
-  soup_uri_free (uri);
-  return TRUE;
-}
-
-/* *** write () *** */
-static void
-write_ready (GObject      *source_object,
-             GAsyncResult *result,
-             gpointer      user_data)
-{
-  GOutputStream *stream;
-  GVfsJob       *job;
-  GError        *error;
-  gssize         nwrote;
-
-  stream = G_OUTPUT_STREAM (source_object); 
-  error  = NULL;
-  job    = G_VFS_JOB (user_data);
-
-  nwrote = g_output_stream_write_finish (stream, result, &error);
-
-  if (nwrote < 0)
-   {
-     g_vfs_job_failed (G_VFS_JOB (job),
-                       error->domain,
-                       error->code,
-                       error->message);
-
-     g_error_free (error);
-     return;
-   }
-
-  g_vfs_job_write_set_written_size (G_VFS_JOB_WRITE (job), nwrote);
-  g_vfs_job_succeeded (job);
-}
-
-static gboolean
-try_write (GVfsBackend *backend,
-           GVfsJobWrite *job,
-           GVfsBackendHandle handle,
-           char *buffer,
-           gsize buffer_size)
-{
-  GVfsBackendHttp *op_backend;
-  GOutputStream   *stream;
-
-  op_backend = G_VFS_BACKEND_HTTP (backend);
-  stream = G_OUTPUT_STREAM (handle);
-
-  g_output_stream_write_async (stream,
-                               buffer,
-                               buffer_size,
-                               G_PRIORITY_DEFAULT,
-                               G_VFS_JOB (job)->cancellable,
-                               write_ready,
-                               job);
-  return TRUE;
-}
-
-/* *** close_write () *** */
-static void
-close_write_ready (GObject      *source_object,
-                   GAsyncResult *result,
-                   gpointer      user_data)
-{
-  GOutputStream *stream;
-  GVfsJob       *job;
-  GError        *error;
-  gboolean       res;
-
-  job = G_VFS_JOB (user_data);
-  stream = G_OUTPUT_STREAM (source_object);
-  res = g_output_stream_close_finish (stream,
-                                      result,
-                                      &error);
-  if (res == FALSE)
-    {
-      g_vfs_job_failed (G_VFS_JOB (job),
-                        error->domain,
-                        error->code,
-                        error->message);
-
-      g_error_free (error);
-    }
-  else
-    g_vfs_job_succeeded (job);
-
-  g_object_unref (stream);
-}
-
-static gboolean
-try_close_write (GVfsBackend *backend,
-                 GVfsJobCloseWrite *job,
-                 GVfsBackendHandle handle)
-{
-  GVfsBackendHttp *op_backend;
-  GOutputStream   *stream;
-
-  op_backend = G_VFS_BACKEND_HTTP (backend);
-  stream = G_OUTPUT_STREAM (handle);
-
-  g_output_stream_close_async (stream,
-                               G_PRIORITY_DEFAULT,
-                               G_VFS_JOB (job)->cancellable,
-                               close_write_ready,
-                               job);
-
-  return TRUE;
-}
 
 /* *** query_info () *** */
+
+static void 
+query_info_ready (SoupSession *session,
+                  SoupMessage *msg,
+                  gpointer     user_data)
+{
+  GFileAttributeMatcher *matcher;
+  GVfsJobQueryInfo      *job;
+  const SoupURI         *uri;
+  const char            *text;
+  GFileInfo             *info;
+  char                  *basename;
+
+  job     = G_VFS_JOB_QUERY_INFO (user_data);
+  info    = job->file_info;
+  matcher = job->attribute_matcher;
+
+  if (! SOUP_STATUS_IS_SUCCESSFUL (msg->status_code))
+    {
+      g_vfs_job_failed_from_http_status (G_VFS_JOB (job), msg->status_code,
+                                         msg->reason_phrase);
+      return;
+    }
+
+  uri = soup_message_get_uri (msg);
+  basename = http_uri_get_basename (uri->path);
+
+  g_print ("basename:%s\n", basename);
+
+  /* read http/1.1 rfc, until then we copy the local files
+   * behaviour */ 
+  if (basename != NULL &&
+      g_file_attribute_matcher_matches (matcher,
+                                        G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME))
+    {
+      char *display_name = g_filename_display_name (basename);
+
+      if (strstr (display_name, "\357\277\275") != NULL)
+        {
+          char *p = display_name;
+          display_name = g_strconcat (display_name, _(" (invalid encoding)"), NULL);
+          g_free (p);
+        }
+
+      g_file_info_set_display_name (info, display_name);
+      g_free (display_name);
+    }
+
+  if (basename != NULL &&
+      g_file_attribute_matcher_matches (matcher,
+                                        G_FILE_ATTRIBUTE_STANDARD_EDIT_NAME))
+    {
+      char *edit_name = g_filename_display_name (basename);
+      g_file_info_set_edit_name (info, edit_name);
+      g_free (edit_name);
+    } 
+
+  g_free (basename);
+
+
+  text = soup_message_headers_get (msg->response_headers,
+                                   "Content-Length");
+  if (text)
+    {
+      guint64 size = g_ascii_strtoull (text, NULL, 10);
+      g_file_info_set_size (info, size);
+    }
+
+
+  text = soup_message_headers_get (msg->response_headers,
+                                   "Content-Type");
+  if (text)
+    {
+      char *p = strchr (text, ';');
+
+      if (p != NULL)
+        {
+          char *tmp = g_strndup (text, p - text);
+          g_file_info_set_content_type (info, tmp);
+          g_free (tmp);
+        }
+      else
+        g_file_info_set_content_type (info, text);
+    }
+
+
+  text = soup_message_headers_get (msg->response_headers,
+                                   "Last-Modified");
+  if (text)
+    {
+      GTimeVal tv;
+      if (g_time_val_from_iso8601 (text, &tv))
+        g_file_info_set_modification_time (info, &tv);
+    }
+
+
+  text = soup_message_headers_get (msg->response_headers,
+                                   "ETag");
+  if (text)
+    {
+      g_file_info_set_attribute_string (info,
+                                        G_FILE_ATTRIBUTE_ETAG_VALUE,
+                                        text);
+    }
+
+
+  g_vfs_job_succeeded (G_VFS_JOB (job));
+}
+
+
 static gboolean
 try_query_info (GVfsBackend           *backend,
                 GVfsJobQueryInfo      *job,
@@ -614,6 +609,14 @@ try_query_info (GVfsBackend           *backend,
                 GFileInfo             *info,
                 GFileAttributeMatcher *attribute_matcher)
 {
+  SoupMessage *msg;
+  SoupURI     *uri;
+
+  uri = http_backend_uri_for_filename (backend, filename, FALSE);
+  msg = soup_message_new_from_uri (SOUP_METHOD_HEAD, uri);
+  soup_uri_free (uri);
+
+  http_backend_queue_message (backend, msg, query_info_ready, job);
 
   return TRUE;
 }
@@ -634,10 +637,6 @@ g_vfs_backend_http_class_init (GVfsBackendHttpClass *klass)
   backend_class->try_read          = try_read;
   backend_class->try_seek_on_read  = try_seek_on_read;
   backend_class->try_close_read    = try_close_read;
-  backend_class->try_create        = try_create;
-  backend_class->try_replace       = try_replace;
-  backend_class->try_write         = try_write;
-  backend_class->try_close_write   = try_close_write;
   backend_class->try_query_info    = try_query_info;
 
 }
